﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Web;
using ScrewTurn.Wiki.PluginFramework;
using System.Web.Security;
using System.Web.Profile;
using System.Configuration;

namespace MembershipUserProvider {

	public class MembershipProvider : IUsersStorageProviderV30 {

		private static log4net.ILog _logger;
		
		private string _config;

		private const string DisplayName = "DisplayName";
		private static readonly string Users = "MembershipUsers";

		#region IUsersStorageProviderV30 Members
		public string ConfigHelpHtml {
			get {
				return string.Empty;
			}
		}

		public ComponentInformation Information {
			get {
				return new ComponentInformation("ASP.NET MembershipUserProvider", "beginor", "v0.1 Beta", "", "");
			}
		}

		public void Init(IHostV30 host, string config) {
			this._config = config;
			if (_logger == null) {
				log4net.Config.XmlConfigurator.ConfigureAndWatch(new FileInfo(HttpContext.Current.Server.MapPath("~/log.config")));
				_logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
			}
			if (_logger.IsDebugEnabled) {
				_logger.DebugFormat("Membership Provider Inited, current provider is {0}.", Membership.Provider);
			}
		}

		public void Shutdown() {
			//throw new NotImplementedException();
			if (_logger.IsDebugEnabled) {
				_logger.Debug("Membership Provider Shutdown.");
			}
		}

		public UserInfo AddUser(string username, string displayName, string password, string email, bool active, DateTime dateTime) {
			var user = Membership.CreateUser(username, password, email);
			user.IsApproved = active;
			SetUserLastActivityDate(user, dateTime);
			Membership.UpdateUser(user);
			SaveDisplayName(user.UserName, displayName);
			return new UserInfo(username, displayName, email, active, dateTime, this);
		}

		private static void SaveProfile(string username, string propertyName, string propertyValue) {
			var userProfile = ProfileBase.Create(username);
			userProfile.SetPropertyValue(propertyName, propertyValue);
			userProfile.Save();
		}

		private static void SaveDisplayName(string username, string displayName) {
			SaveProfile(username, DisplayName, displayName);
		}

		private static object GetProfile(string username, string propertyName) {
			var userProfile = ProfileBase.Create(username);
			return userProfile.GetPropertyValue(propertyName);
		}

		private static string GetDisplayName(string username) {
			object val = GetProfile(username, DisplayName);
			return val == null ? string.Empty : val.ToString();
		}

		public UserGroup AddUserGroup(string name, string description) {
			var group = new UserGroup(name, string.Empty, this);
			Roles.CreateRole(name);
			return group;
		}

		public UserInfo GetUser(string username) {
			var member = Membership.GetUser(username);
			if (member == null) {
				return null;
			}
			var user = new UserInfo(username, GetDisplayName(username), member.Email, member.IsApproved, GetUserLastActivityDate(member), this);
			user.Groups = GetUserGroups(user.Username);
			user.Provider = this;
			return user;
		}

		public UserInfo GetUserByEmail(string email) {
			var username = Membership.GetUserNameByEmail(email);
			if (string.IsNullOrEmpty(username)) {
				return null;
			}
			return this.GetUser(username);
		}

		public UserGroup[] GetUserGroups() {
			var roles = Roles.GetAllRoles();
			var groups = new UserGroup[roles.Length];
			for (int i = 0; i < roles.Length; i++) {
				groups[i] = new UserGroup(roles[i], roles[i], this) {
					Users = Roles.GetUsersInRole(roles[i])
				};
			}
			return groups;
		}

		public UserInfo[] GetUsers() {
			var users = Membership.GetAllUsers();
			var userArr = new MembershipUser[users.Count];
			users.CopyTo(userArr, 0);
			var result = new UserInfo[userArr.Length];
			for (int i = 0; i < userArr.Length; i++) {
				result[i] = new UserInfo(userArr[i].UserName, string.Empty, userArr[i].Email, userArr[i].IsApproved, GetUserLastActivityDate(userArr[i]), this);
				result[i].Groups = GetUserGroups(result[i].Username);
			}
			return result;
		}

		private static string[] GetUserGroups(string username) {
			var userRoles = Roles.GetRolesForUser(username);
			if (userRoles == null || userRoles.Length == 0) {
				Roles.AddUserToRole(username, Users);
			}
			return Roles.GetRolesForUser(username);
		}

		private static DateTime GetUserLastActivityDate(MembershipUser user) {
			if (user is ActiveDirectoryMembershipUser) {
				return DateTime.Now;
			}
			return user.LastActivityDate;
		}

		private static void SetUserLastActivityDate(MembershipUser user, DateTime lastActivityDate) {
			if (user is ActiveDirectoryMembershipUser) {
				return;
			}
			user.LastActivityDate = lastActivityDate;
		}

		public IDictionary<UserInfo, string> GetUsersWithData(string key) {
			var users = this.GetUsers();
			var result = new Dictionary<UserInfo, string>();
			foreach (var user in users) {
				result[user] = this.RetrieveUserData(user, key);
			}
			return result;
		}

		public bool GroupMembershipReadOnly {
			get {
				return true;
			}
		}

		public UserInfo ModifyUser(UserInfo user, string newDisplayName, string newPassword, string newEmail, bool newActive) {
			var member = Membership.GetUser(user.Username);
			if (member == null) {
				return null;
			}
			SaveDisplayName(member.UserName, newDisplayName);

			member.ChangePassword(member.ResetPassword(), newPassword);
			member.Email = newEmail;
			member.IsApproved = newActive;

			Membership.UpdateUser(member);

			return this.GetUser(member.UserName);
		}

		public UserGroup ModifyUserGroup(UserGroup group, string description) {
			return group;
		}

		/// <summary>
		/// Notifies the provider that a user has logged in through the authentication cookie.
		/// </summary>
		/// <param name="user">The user who has logged in.</param>
		/// <exception cref="ArgumentNullException">If <paramref name="user"/> is <c>null</c>.</exception>
		public void NotifyCookieLogin(UserInfo user) {
			if (user == null) {
				throw new ArgumentNullException("user");
			}
			if (_logger.IsDebugEnabled) {
				_logger.Debug("a user has logged in through auth cookie, check to update the auth ticket;");
			}
			HttpCookie authCookie = HttpContext.Current.Request.Cookies[FormsAuthentication.FormsCookieName];
			if (authCookie != null) {
				FormsAuthenticationTicket ticket = FormsAuthentication.Decrypt(authCookie.Value);
				FormsAuthentication.RenewTicketIfOld(ticket);
			}
		}

		public void NotifyLogout(UserInfo user) {
			if (_logger.IsDebugEnabled) {
				_logger.DebugFormat("Logout User {0}.", user.Username);
			}
			FormsAuthentication.SignOut();
		}

		public bool RemoveUser(UserInfo user) {
			var userRoles = Roles.GetRolesForUser(user.Username);
			Roles.RemoveUserFromRoles(user.Username, userRoles);
			return Membership.DeleteUser(user.Username);
		}

		public bool RemoveUserGroup(UserGroup group) {
			return Roles.DeleteRole(group.Name);
		}

		public IDictionary<string, string> RetrieveAllUserData(UserInfo user) {
			var userProfile = ProfileBase.Create(user.Username);
			var userData = new Dictionary<string, string>();
			foreach (SettingsProperty property in ProfileBase.Properties) {
				userData[property.Name] = userProfile.GetPropertyValue(property.Name).ToString();
			}
			return userData;
		}

		public string RetrieveUserData(UserInfo user, string key) {
			var val = GetProfile(user.Username, key);
			return val == null ? string.Empty : val.ToString();
		}

		public UserInfo SetUserMembership(UserInfo user, string[] groups) {
			Roles.AddUserToRoles(user.Username, groups);
			return user;
		}

		public bool StoreUserData(UserInfo user, string key, string value) {
			try {
				SaveProfile(user.Username, key, value);
				return true;
			}
			catch (Exception ex) {
				return false;
			}
		}

		public bool TestAccount(UserInfo user, string password) {
			if (_logger.IsDebugEnabled) {
				_logger.DebugFormat("Test user {0} with password {1}", user.Username, password);
			}
			return Membership.ValidateUser(user.Username, password);
		}

		/// <summary>
		/// Tries to login a user directly through the provider using 
		/// the current HttpContext and without username/password.
		/// </summary>
		/// <param name="context">The current HttpContext.</param>
		/// <returns>The correct UserInfo object, or <c>null</c>.</returns>
		/// <exception cref="ArgumentNullException">If <paramref name="context"/> is <c>null</c>.</exception>
		public UserInfo TryAutoLogin(HttpContext context) {
			if (_logger.IsDebugEnabled) {
				_logger.Debug("Tries to login a user directly through the provider using the current HttpContext and without username/password.");
			}
			if (context.User.Identity.IsAuthenticated) {
				if (_logger.IsDebugEnabled) {
					_logger.DebugFormat("Current User is authenticated, user name is {0}, login directly.", context.User.Identity.Name);
				}
				return GetUser(context.User.Identity.Name);
			}
			return null;
		}

		public UserInfo TryManualLogin(string username, string password) {
			if (_logger.IsDebugEnabled) {
				_logger.DebugFormat("Try manual login user {0}.", username);
			}
			var isValid = Membership.ValidateUser(username, password);
			if (isValid) {
				if (_logger.IsDebugEnabled) {
					_logger.Debug("The username and password is valid, set forms auth ticket and login.");
				}
				FormsAuthentication.SetAuthCookie(username, true);
				return this.GetUser(username);
			}
			if (_logger.IsDebugEnabled) {
				_logger.Debug("The username and password is invalid, cannot login.");
			}
			return null;
		}

		public bool UserAccountsReadOnly {
			get {
				return true;
			}
		}

		public bool UserGroupsReadOnly {
			get {
				return true;
			}
		}

		public bool UsersDataReadOnly {
			get {
				return false;
			}
		}
		#endregion
	}
}
